@@ -116,6 +116,7 @@ namespace hfst
         assert(false);
         break;
       }
+    HFST_THROW(HfstFatalException); // make compiler happy
   }
 
   short &HfstInputStream::stream_get(short &i)
@@ -162,6 +163,7 @@ namespace hfst
         assert(false);
         break;
       }
+    HFST_THROW(HfstFatalException); // make compiler happy
   }
 
   unsigned short &HfstInputStream::stream_get(unsigned short &i)
@@ -212,6 +214,7 @@ namespace hfst
         assert(false);
         break;
       }
+    HFST_THROW(HfstFatalException); // make compiler happy
   }
 
   void HfstInputStream::stream_unget(char c)
@@ -305,25 +308,28 @@ namespace hfst
 
   void HfstInputStream::read_transducer(HfstTransducer &t)
   {
-    if (input_stream != NULL) { // first transducer in the stream
-      input_stream = NULL;
-      if (stream_eof())
-        HFST_THROW(EndOfStreamException);
-      // if header bytes have been read from a file, skip these bytes
-      if (strcmp(filename.c_str(), "") != 0)
-        ignore(bytes_to_skip);
-    }
-    else {
-      if (stream_eof())
-        HFST_THROW(EndOfStreamException);
-      ImplementationType current_type  = this->get_type();
-      ImplementationType stype = this->stream_fst_type();
-      if (stype != current_type) {
-        HFST_THROW_MESSAGE(TransducerTypeMismatchException,
-                           "HfstInputStream contains HfstTransducers "
-                           "whose type is not the same");
+    if (type != XFSM_TYPE)
+      {
+        if (input_stream != NULL) { // first transducer in the stream
+          input_stream = NULL;
+          if (stream_eof())
+            HFST_THROW(EndOfStreamException);
+          // if header bytes have been read from a file, skip these bytes
+          if (strcmp(filename.c_str(), "") != 0)
+            ignore(bytes_to_skip);
+        }
+        else {
+          if (stream_eof())
+            HFST_THROW(EndOfStreamException);
+          ImplementationType current_type  = this->get_type();
+          ImplementationType stype = this->stream_fst_type();
+          if (stype != current_type) {
+            HFST_THROW_MESSAGE(TransducerTypeMismatchException,
+                               "HfstInputStream contains HfstTransducers "
+                               "whose type is not the same");
+          }
+        }
       }
-    }
 
     switch (type)
       {
@@ -365,7 +371,7 @@ namespace hfst
              and/or that epsilon is coded differently than "@_EPSILON_SYMBOL_@"
              and/or that numbers 1 and 2 are reserved for other use than
              "@_UNKNOWN_SYMBOL_@" or "@_IDENTITY_SYMBOL_@". */
-          if (not has_hfst_header)
+          if (! has_hfst_header)
             {
               HfstBasicTransducer * net = 
                 ConversionFunctions::
@@ -481,7 +487,7 @@ namespace hfst
              and/or that epsilon is coded differently than "@_EPSILON_SYMBOL_@"
              and/or that numbers 1 and 2 are reserved for other use than
              "@_UNKNOWN_SYMBOL_@" or "@_IDENTITY_SYMBOL_@". */
-          if (not has_hfst_header)
+          if (! has_hfst_header)
             {
               HfstBasicTransducer * net = 
                 ConversionFunctions::
@@ -496,8 +502,9 @@ namespace hfst
 
         if (hfst_version_2_weighted_transducer) // this should not happen
           { 
-            fprintf(stderr, "ERROR: not transducer stream\n");
-            exit(1);
+            //fprintf(stderr, "ERROR: not transducer stream\n");
+            //exit(1);
+            HFST_THROW_MESSAGE(HfstFatalException, "not transducer stream");
           }
         break;
         }
@@ -509,6 +516,12 @@ namespace hfst
           this->implementation.foma->read_transducer();
         break;
 #endif
+#if HAVE_XFSM
+      case XFSM_TYPE:
+        t.implementation.xfsm =
+          this->implementation.xfsm->read_transducer();
+        break;
+#endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
       case MY_TRANSDUCER_LIBRARY_TYPE:
         t.implementation.my_transducer_library =
@@ -529,13 +542,16 @@ namespace hfst
         break;
   }
 
-  t.set_name(name);
-  for (std::map<string,string>::const_iterator prop = props.begin();
-       prop != props.end();
-       ++prop)
-    {
-      t.set_property(prop->first, prop->second);
-    }
+    if (type != XFSM_TYPE)
+      {
+        t.set_name(name);
+        for (std::map<string,string>::const_iterator prop = props.begin();
+             prop != props.end();
+             ++prop)
+          {
+            t.set_property(prop->first, prop->second);
+          }
+      }
 
 }
 
@@ -619,7 +635,12 @@ namespace hfst
           break;
         }
       default:
+#ifdef HAVE_XFSM
+        return XFSM_;
+#else
         return ERROR_TYPE_;
+#endif
+
       }
     return ERROR_TYPE_;
   }
@@ -633,7 +654,7 @@ namespace hfst
     }
 
     // (1) first pair "version", "3.0"
-    if ( not ( ( strcmp("version", header_data[0].first.c_str()) == 0 ) &&
+    if ( ! ( ( strcmp("version", header_data[0].first.c_str()) == 0 ) &&
                ( ( strcmp("3.0", header_data[0].second.c_str()) == 0 ) ||
                  ( strcmp("3.3", header_data[0].second.c_str()) == 0 ) ) ) ){
       HFST_THROW_MESSAGE(TransducerHeaderException,
@@ -641,7 +662,7 @@ namespace hfst
     }
 
     // (2) second pair "type", (valid type field)
-    if ( not ( strcmp("type", header_data[1].first.c_str()) == 0 ) ) {
+    if ( ! ( strcmp("type", header_data[1].first.c_str()) == 0 ) ) {
       HFST_THROW_MESSAGE(TransducerHeaderException,
                          "Hfst header: transducer type not given");
     }
@@ -857,7 +878,7 @@ namespace hfst
     }
 
     // whether the stream contains an HFST version <3.0 transducer
-    // or an SFST, OpenFst or a foma transducer
+    // or a native SFST, OpenFst, foma or xfsm transducer
     TransducerType transducer_type = guess_fst_type(bytes_read);
     bytes_to_skip=bytes_read;
 
@@ -888,6 +909,9 @@ namespace hfst
       case FOMA_:
         return FOMA_TYPE;
         break;
+      case XFSM_:
+        return XFSM_TYPE;
+        break;
 #if HAVE_MY_TRANSDUCER_LIBRARY
       case MY_TRANSDUCER_LIBRARY_:
         return MY_TRANSDUCER_LIBRARY_TYPE;
@@ -916,7 +940,7 @@ namespace hfst
     catch (const HfstException e)
     { throw e; }
 
-    if ( not HfstTransducer::is_implementation_type_available(type)) {
+    if ( ! HfstTransducer::is_implementation_type_available(type)) {
       HFST_THROW(ImplementationTypeNotAvailableException);
     }
 
@@ -944,6 +968,12 @@ namespace hfst
       implementation.foma = new hfst::implementations::FomaInputStream;
       break;
 #endif
+#if HAVE_XFSM
+    case XFSM_TYPE:
+      // Will throw an error.
+      implementation.xfsm = new hfst::implementations::XfsmInputStream;
+      break;
+#endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
     case MY_TRANSDUCER_LIBRARY_TYPE:
       implementation.my_transducer_library 
@@ -992,7 +1022,7 @@ namespace hfst
 
     catch( const HfstException e)
       { throw e; }
-    if ( not HfstTransducer::is_implementation_type_available(type)) {
+    if ( ! HfstTransducer::is_implementation_type_available(type)) {
       HFST_THROW(ImplementationTypeNotAvailableException);
     }
 
@@ -1028,6 +1058,12 @@ namespace hfst
         = new hfst::implementations::FomaInputStream(filename);
       break;
 #endif
+#if HAVE_XFSM
+    case XFSM_TYPE:
+      implementation.xfsm 
+        = new hfst::implementations::XfsmInputStream(filename);
+      break;
+#endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
     case MY_TRANSDUCER_LIBRARY_TYPE:
       implementation.my_transducer_library 
@@ -1074,6 +1110,11 @@ namespace hfst
         delete implementation.foma;
         break;
 #endif
+#if HAVE_XFSM
+      case XFSM_TYPE:
+        delete implementation.xfsm;
+        break;
+#endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
       case MY_TRANSDUCER_LIBRARY_TYPE:
         delete implementation.my_transducer_library;
@@ -1114,6 +1155,11 @@ namespace hfst
         implementation.foma->close();
         break;
 #endif
+#if HAVE_XFSM
+      case XFSM_TYPE:
+        implementation.xfsm->close();
+        break;
+#endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
       case MY_TRANSDUCER_LIBRARY_TYPE:
         implementation.my_transducer_library->close();
@@ -1152,6 +1198,11 @@ namespace hfst
         return implementation.foma->is_eof();
         break;
 #endif
+#if HAVE_XFSM
+      case XFSM_TYPE:
+        return implementation.xfsm->is_eof();
+        break;
+#endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
       case MY_TRANSDUCER_LIBRARY_TYPE:
         return implementation.my_transducer_library->is_eof();
@@ -1190,6 +1241,11 @@ namespace hfst
         return implementation.foma->is_bad();
         break;
 #endif
+#if HAVE_XFSM
+      case XFSM_TYPE:
+        return implementation.xfsm->is_bad();
+        break;
+#endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
       case MY_TRANSDUCER_LIBRARY_TYPE:
         return implementation.my_transducer_library->is_bad();
@@ -1229,6 +1285,11 @@ namespace hfst
         return implementation.foma->is_good();
         break;
 #endif
+#if HAVE_XFSM
+      case XFSM_TYPE:
+        return implementation.xfsm->is_good();
+        break;
+#endif
 #if HAVE_MY_TRANSDUCER_LIBRARY
       case MY_TRANSDUCER_LIBRARY_TYPE:
         return implementation.my_transducer_library->is_good();
