@@ -77,7 +77,7 @@ namespace hfst { namespace implementations {
   
   bool FomaInputStream::is_good(void)
   {
-    return not is_bad();
+    return ! is_bad();
   };
   
   bool FomaInputStream::is_fst(void)
@@ -525,7 +525,7 @@ namespace hfst { namespace implementations {
               }
           }
         
-        hfst::HfstTwoLevelPath path(0, spv);
+        hfst::HfstTwoLevelPath path(float(0), spv);
         hfst::ExtractStringsCb::RetVal ret = callback(path, final);
         if(!ret.continueSearch || !ret.continuePath)
           {
@@ -671,7 +671,7 @@ namespace hfst { namespace implementations {
           }
         if (final_initial) {
           StringPairVector empty_spv;
-          HfstTwoLevelPath epsilon_path(0, empty_spv);
+          HfstTwoLevelPath epsilon_path(float(0), empty_spv);
           callback(epsilon_path, true /* final*/);
         }
       }
@@ -798,6 +798,7 @@ namespace hfst { namespace implementations {
     return table;
   }
 
+#if GENERATE_LEXC_WRAPPER
     fsm * FomaTransducer::read_lexc(const std::string &filename, bool verbose)
   {
     char * filename_ = strdup(filename.c_str());
@@ -822,6 +823,7 @@ namespace hfst { namespace implementations {
     free(lexcfile);
     return retval;
   }
+#endif
 
   struct fsm * FomaTransducer::eliminate_flags(struct fsm * t)
   {
@@ -849,7 +851,7 @@ namespace hfst { namespace implementations {
     /* Read foma transducer . */
     struct fsm * FomaTransducer::read_net(FILE *infile) {
       
-    unsigned int READ_BUF_SIZE=4096; 
+    const unsigned int READ_BUF_SIZE=4096; 
     char buf[READ_BUF_SIZE];
     struct fsm *net;
     struct fsm_state *fsm;
