@@ -13,12 +13,29 @@
 #include "TropicalWeightTransducer.h"
 #include "HfstSymbolDefs.h"
 #include "HfstLookupFlagDiacritics.h"
+#include "HfstTransitionGraph.h"
+#include "ConvertTransducerFormat.h"
 
 #ifndef MAIN_TEST
+
+#define CHECK_EPSILON_CYCLES(x, y) { hfst::implementations::HfstBasicTransducer * fsm = hfst::implementations::ConversionFunctions::tropical_ofst_to_hfst_basic_transducer( x ); if (fsm->has_negative_epsilon_cycles()) { if (warning_stream != NULL) { *warning_stream << y << ": warning: transducer has epsilon cycles with a negative weight" << std::endl; } } delete fsm; }
+
 namespace hfst { 
   bool get_encode_weights();
 
   namespace implementations {
+
+    std::ostream * TropicalWeightTransducer::warning_stream = NULL;
+
+    std::ostream * TropicalWeightTransducer::get_warning_stream()
+    {
+      return warning_stream;
+    }
+
+    void TropicalWeightTransducer::set_warning_stream(std::ostream * os)
+    {
+      warning_stream = os;
+    }
     
     // This function can be moved to its own file if TropicalWeightTransducer.o
     // yields a 'File too big' error.
@@ -26,6 +43,9 @@ namespace hfst {
     (StdVectorFst * t, bool to_initial_state)
     {
       assert (t->InputSymbols() != NULL);
+
+      CHECK_EPSILON_CYCLES(t, "push_weights");
+      
       fst::StdVectorFst * retval = new fst::StdVectorFst();
       if (to_initial_state)
         fst::Push<StdArc, REWEIGHT_TO_INITIAL>(*t, retval, fst::kPushWeights);
@@ -35,12 +55,76 @@ namespace hfst {
       return retval;
     }
 
+    void TropicalWeightTransducer::add_to_weights(StdVectorFst * t, float w)
+    {
+      for (fst::StateIterator<StdVectorFst> siter(*t); 
+           ! siter.Done(); siter.Next()) 
+        {
+          StateId s = siter.Value();
+          for (fst::MutableArcIterator<fst::StdVectorFst> aiter(t,s); 
+               !aiter.Done(); aiter.Next())
+            {
+              const StdArc &arc = aiter.Value();
+
+              StdArc new_arc;
+              new_arc.ilabel = arc.ilabel;
+              new_arc.olabel = arc.olabel;
+              new_arc.nextstate = arc.nextstate;
+              new_arc.weight = arc.weight.Value() + w;
+
+              aiter.SetValue(new_arc);
+            }
+          if (t->Final(s) != TropicalWeight::Zero())
+            {
+              float old_weight = t->Final(s).Value();
+              t->SetFinal(s, old_weight + w);
+            }
+        }
+    }
+
+    float TropicalWeightTransducer::get_smallest_weight(StdVectorFst * t)
+    {
+      // in case of an empty transducer, infinity is returned
+      // (empty in the sense of having no transitions or final states)
+      float retval = std::numeric_limits<float>::infinity();
+      bool weight_found;
+      for (fst::StateIterator<StdVectorFst> siter(*t); 
+           ! siter.Done(); siter.Next()) 
+        {
+          StateId s = siter.Value();
+          for (fst::ArcIterator<StdVectorFst> aiter(*t,s); 
+               !aiter.Done(); aiter.Next())
+            {
+              const StdArc &arc = aiter.Value();
+              float w = arc.weight.Value();
+              if (w < retval)
+                retval = w;
+            }
+          if (t->Final(s) != TropicalWeight::Zero())
+            {
+              float w = t->Final(s).Value();
+              if (w < retval)
+                retval = w;
+            }
+        }
+      return retval;
+    }
+
     // This function can be moved to its own file if TropicalWeightTransducer.o
     // yields a 'File too big' error.
     StdVectorFst * TropicalWeightTransducer::minimize(StdVectorFst * t)
     {
+
+      CHECK_EPSILON_CYCLES(t, "minimize");
+
       RmEpsilon<StdArc>(t);
 
+      float w = get_smallest_weight(t); 
+      if (w < 0) 
+        { 
+          add_to_weights(t, -w); 
+        }
+
       EncodeMapper<StdArc> encode_mapper
         (hfst::get_encode_weights() ? (kEncodeLabels|kEncodeWeights) : (kEncodeLabels), ENCODE);
       Encode(t, &encode_mapper);
@@ -49,6 +133,12 @@ namespace hfst {
       Determinize<StdArc>(*t, det);
       Minimize<StdArc>(det);
       Decode(det, encode_mapper);
+
+      if (w < 0) 
+        { 
+          add_to_weights(det, w); 
+        }
+
       return det;
     }
 
@@ -111,7 +201,7 @@ namespace hfst {
     fst::SymbolTable st(t->InputSymbols()->Name());    
     for ( fst::SymbolTableIterator it 
             = fst::SymbolTableIterator(*(t->InputSymbols()));
-          not it.Done(); it.Next() ) {
+           !it.Done(); it.Next() ) {
       if (it.Symbol() != symbol) {
     st.AddSymbol(it.Symbol(), it.Value());
       }
@@ -158,7 +248,7 @@ namespace hfst {
     StringSet s;
     for ( fst::SymbolTableIterator it 
             = fst::SymbolTableIterator(*(t->InputSymbols()));
-          not it.Done(); it.Next() ) {
+          ! it.Done(); it.Next() ) {
       s.insert( std::string(it.Symbol()) );
     }
     return s;
@@ -181,7 +271,7 @@ namespace hfst {
     unsigned int biggest_number=0;
     for ( fst::SymbolTableIterator it 
             = fst::SymbolTableIterator(*(t->InputSymbols()));
-          not it.Done(); it.Next() ) {
+          ! it.Done(); it.Next() ) {
       if (it.Value() > biggest_number) 
         biggest_number = it.Value();
     }      
@@ -216,12 +306,12 @@ namespace hfst {
     // find the number-to-number mappings for transducer t1
     for ( fst::SymbolTableIterator it 
             = fst::SymbolTableIterator(*(t1->InputSymbols()));
-          not it.Done(); it.Next() ) {    
+          ! it.Done(); it.Next() ) {    
       km [ (unsigned int)it.Value() ] 
         = (unsigned int) t2->InputSymbols()->Find( it.Symbol() );
       
       assert(it.Value() >= 0);
-      assert(not (t2->InputSymbols()->Find( it.Symbol() ) < 0));
+      assert(! (t2->InputSymbols()->Find( it.Symbol() ) < 0));
     }
     
     return km;
@@ -232,7 +322,7 @@ namespace hfst {
     (StdVectorFst *t, NumberNumberMap &km) 
   {
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         StateId s = siter.Value();
         for (fst::MutableArcIterator<StdVectorFst> aiter(t,s); 
@@ -258,7 +348,7 @@ namespace hfst {
   (StdVectorFst * t, float weight, bool increment)
   {
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         StateId s = siter.Value();
         if ( t->Final(s) != TropicalWeight::Zero() )
@@ -295,7 +385,7 @@ namespace hfst {
     (StdVectorFst * t,float (*func)(float f))
   {
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         StateId s = siter.Value();
         if ( t->Final(s) != TropicalWeight::Zero() )
@@ -333,7 +423,7 @@ namespace hfst {
     }
       
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next()) 
+         ! siter.Done(); siter.Next()) 
       {
         StateId s = siter.Value();
         if (s == initial_state) {
@@ -367,7 +457,7 @@ namespace hfst {
       }
 
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         StateId s = siter.Value();
         if (s != initial_state) {
@@ -416,7 +506,7 @@ namespace hfst {
     }
       
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next()) 
+         ! siter.Done(); siter.Next()) 
       {
         StateId s = siter.Value();
         if (s == initial_state) {
@@ -449,7 +539,7 @@ namespace hfst {
       }
 
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         StateId s = siter.Value();
         if (s != initial_state) {
@@ -500,7 +590,7 @@ namespace hfst {
     }
       
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next()) 
+         ! siter.Done(); siter.Next()) 
       {
         StateId s = siter.Value();
         if (s == initial_state) {
@@ -536,7 +626,7 @@ namespace hfst {
       }
 
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         StateId s = siter.Value();
         if (s != initial_state) {
@@ -586,7 +676,7 @@ namespace hfst {
     }
       
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next()) 
+         ! siter.Done(); siter.Next()) 
       {
         StateId s = siter.Value();
         if (s == initial_state) {
@@ -622,7 +712,7 @@ namespace hfst {
       }
 
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         StateId s = siter.Value();
         if (s != initial_state) {
@@ -766,12 +856,12 @@ namespace hfst {
     StdVectorFst * result = new StdVectorFst();
 
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       result->AddState();
 
     // go through all states in this
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         // create new state in result, if needed
         StateId s = siter.Value();
@@ -808,13 +898,13 @@ namespace hfst {
                 for (StringSet::iterator it1 = unknown.begin(); 
                      it1 != unknown.end(); it1++) 
                   {
-            if (not FdOperation::is_diacritic(*it1)) {
+            if (! FdOperation::is_diacritic(*it1)) {
               
               int64 inumber = is->Find(*it1);
               for (StringSet::iterator it2 = unknown.begin(); 
                it2 != unknown.end(); it2++) 
             {
-              if (not FdOperation::is_diacritic(*it2)) {
+              if (! FdOperation::is_diacritic(*it2)) {
                 int64 onumber = is->Find(*it2);
                 if (inumber != onumber)
                   result->AddArc(result_s, 
@@ -838,7 +928,7 @@ namespace hfst {
                 for (StringSet::iterator it = unknown.begin(); 
                      it != unknown.end(); it++) 
                   {
-            if (not FdOperation::is_diacritic(*it)) {
+            if (! FdOperation::is_diacritic(*it)) {
               int64 number = is->Find(*it);
               result->AddArc(result_s, 
                      StdArc(number, number, 
@@ -851,7 +941,7 @@ namespace hfst {
                 for (StringSet::iterator it = unknown.begin(); 
                      it != unknown.end(); it++) 
                   {
-            if (not FdOperation::is_diacritic(*it)) {
+            if (! FdOperation::is_diacritic(*it)) {
               int64 number = is->Find(*it);
               result->AddArc(result_s, 
                      StdArc(number, arc.olabel, 
@@ -864,7 +954,7 @@ namespace hfst {
                 for (StringSet::iterator it = unknown.begin(); 
                      it != unknown.end(); it++) 
                   {
-            if (not FdOperation::is_diacritic(*it)) {
+            if (! FdOperation::is_diacritic(*it)) {
               int64 number = is->Find(*it);
               result->AddArc(result_s, 
                      StdArc(arc.ilabel, number, 
@@ -889,12 +979,12 @@ namespace hfst {
     StdVectorFst * result = new StdVectorFst();
 
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       result->AddState();
     
     // go through all states in this
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         // create new state in result, if needed
         StateId s = siter.Value();
@@ -931,7 +1021,7 @@ namespace hfst {
   {
     unsigned int retval=0;
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       retval++;
     return retval;
   }
@@ -940,7 +1030,7 @@ namespace hfst {
   {
     unsigned int retval=0;
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         for (fst::ArcIterator<StdVectorFst> aiter(*t,siter.Value()); 
              !aiter.Done(); aiter.Next())
@@ -1013,14 +1103,14 @@ namespace hfst {
     fst::StdVectorFst *harmonized_t1;
     fst::StdVectorFst *harmonized_t2;
 
-    if (not unknown_symbols_in_use)
+    if (! unknown_symbols_in_use)
       harmonized_t1 = t1;
     else {
       harmonized_t1 = expand_arcs(t1, unknown_t1, unknown_symbols_in_use);
       harmonized_t1->SetInputSymbols(t1->InputSymbols());
     }
 
-    if (not unknown_symbols_in_use)
+    if (! unknown_symbols_in_use)
       harmonized_t2 = t2;
     else {
       harmonized_t2 = expand_arcs(t2, unknown_t2, unknown_symbols_in_use);
@@ -1284,7 +1374,7 @@ namespace hfst {
   (const std::string &symbol)
   {
 
-    assert(not (symbol == ""));
+    assert(! (symbol == ""));
 
     StdVectorFst * t = new StdVectorFst;
     SymbolTable st = create_symbol_table("");
@@ -1303,8 +1393,8 @@ namespace hfst {
     (const std::string &isymbol, const std::string &osymbol)
   {
 
-    assert(not (isymbol == ""));
-    assert(not (osymbol == ""));
+    assert(! (isymbol == ""));
+    assert(! (osymbol == ""));
 
     StdVectorFst * t = new StdVectorFst;
     SymbolTable st = create_symbol_table("");
@@ -1325,6 +1415,9 @@ namespace hfst {
     StdVectorFst * a = copy(a_);
     StdVectorFst * b = copy(b_);
 
+    CHECK_EPSILON_CYCLES(a, "are_equivalent");
+    CHECK_EPSILON_CYCLES(b, "are_equivalent");
+
     RmEpsilon<StdArc>(a);
     RmEpsilon<StdArc>(b);
 
@@ -1354,7 +1447,7 @@ namespace hfst {
   bool TropicalWeightTransducer::is_automaton(StdVectorFst * t)
   {
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next()) 
+         ! siter.Done(); siter.Next()) 
       {
         StateId s = siter.Value();
         for (fst::ArcIterator<StdVectorFst> aiter(*t,s); 
@@ -1390,8 +1483,8 @@ namespace hfst {
       {
         StateId s2 = t->AddState();
 
-    assert(not (it->first == ""));
-    assert(not (it->second == ""));
+    assert(! (it->first == ""));
+    assert(! (it->second == ""));
 
         t->AddArc(s1,StdArc(st.AddSymbol(it->first),
                             st.AddSymbol(it->second),0,s2));
@@ -1412,15 +1505,15 @@ namespace hfst {
     t->SetStart(s1);
     StateId s2=s1;               // final state
 
-    if (not sps.empty()) {
-      if (not cyclic)
+    if (! sps.empty()) {
+      if (! cyclic)
         s2 = t->AddState();
       for (StringPairSet::const_iterator it = sps.begin();
            it != sps.end();
            ++it)
         {
-      assert(not (it->first == ""));
-      assert(not (it->second == ""));
+      assert(! (it->first == ""));
+      assert(! (it->second == ""));
 
           t->AddArc(s1,StdArc(st.AddSymbol(it->first),
                               st.AddSymbol(it->second),0,s2));
@@ -1447,8 +1540,8 @@ namespace hfst {
         for (StringPairSet::const_iterator it2 = (*it).begin(); 
              it2 != (*it).end(); it2++ ) {
 
-      assert(not (it2->first == ""));
-      assert(not (it2->second == ""));
+      assert(! (it2->first == ""));
+      assert(! (it2->second == ""));
 
           t->AddArc(s1,StdArc(st.AddSymbol(it2->first),
                               st.AddSymbol(it2->second),0,s2));
@@ -1489,8 +1582,8 @@ namespace hfst {
     t->SetStart(s1);
     StateId s2=s1;               // final state
 
-    if (not nps.empty()) {
-      if (not cyclic)
+    if (! nps.empty()) {
+      if (! cyclic)
         s2 = t->AddState();
       for (NumberPairSet::const_iterator it = nps.begin();
            it != nps.end();
@@ -1534,14 +1627,29 @@ namespace hfst {
   StdVectorFst * 
   TropicalWeightTransducer::determinize(StdVectorFst * t)
   {
+
+    CHECK_EPSILON_CYCLES(t, "determinize");
+
     RmEpsilon<StdArc>(t);
 
+    float w = get_smallest_weight(t); 
+    if (w < 0) 
+      { 
+        add_to_weights(t, -w); 
+      }
+
     EncodeMapper<StdArc> encode_mapper
       (hfst::get_encode_weights() ? (kEncodeLabels|kEncodeWeights) : (kEncodeLabels), ENCODE);
     Encode(t, &encode_mapper);
     StdVectorFst * det = new StdVectorFst();
     Determinize<StdArc>(*t, det);
     Decode(det, encode_mapper);
+
+    if (w < 0) 
+      { 
+        add_to_weights(det, w); 
+      }
+
     return det;
   }
 
@@ -1577,7 +1685,7 @@ namespace hfst {
   void print_att_number(StdVectorFst *t, FILE * ofile) {
     fprintf(ofile, "initial state: %i\n", t->Start());
     for (fst::StateIterator<StdVectorFst> siter(*t); 
-         not siter.Done(); siter.Next()) 
+         ! siter.Done(); siter.Next()) 
       {
         StateId s = siter.Value();
         if ( t->Final(s) != TropicalWeight::Zero() )
@@ -1682,7 +1790,9 @@ namespace hfst {
 
   StdVectorFst * 
   TropicalWeightTransducer::remove_epsilons(StdVectorFst * t)
-  { return new StdVectorFst(RmEpsilonFst<StdArc>(*t)); }
+  {
+    CHECK_EPSILON_CYCLES(t, "remove_epsilons");
+    return new StdVectorFst(RmEpsilonFst<StdArc>(*t)); }
 
   StdVectorFst * 
   TropicalWeightTransducer::prune(StdVectorFst * t)
@@ -1694,15 +1804,29 @@ namespace hfst {
   StdVectorFst * 
   TropicalWeightTransducer::n_best(StdVectorFst * t, unsigned int n)
   { 
+    CHECK_EPSILON_CYCLES(t, "n_best");
+
     StdVectorFst * n_best_fst = new StdVectorFst(); 
+    StdVectorFst * scaled = t->Copy();
+    RmEpsilon(scaled);
+    float w = get_smallest_weight(scaled); 
+    if (w < 0) 
+      { 
+        add_to_weights(scaled, -w); 
+      }
     try 
       {
-        fst::ShortestPath(*t,n_best_fst,(size_t)n);
+        fst::ShortestPath(*scaled,n_best_fst,(size_t)n);
       }
     catch (const std::bad_alloc & e)
       {
         HFST_THROW_MESSAGE(HfstFatalException, "TropicalWeightTransducer::nbest runs out of memory");
       }
+    RmEpsilon(n_best_fst);
+    if (w < 0) 
+      { 
+        add_to_weights(n_best_fst, w); 
+      }
     return n_best_fst;
   }
 
@@ -1878,7 +2002,7 @@ namespace hfst {
     fst::SymbolTable * st = tc->InputSymbols()->Copy();
     assert(st != NULL);
     for (fst::StateIterator<fst::StdVectorFst> siter(*tc); 
-         not siter.Done(); siter.Next()) 
+         ! siter.Done(); siter.Next()) 
       {
         StateId s = siter.Value();
         for (fst::MutableArcIterator<StdVectorFst> aiter(tc,s); 
@@ -1936,12 +2060,13 @@ namespace hfst {
   {
     assert(t->InputSymbols() != NULL);
     SymbolTable * st = t->InputSymbols()->Copy();
+    // TODO: cl.exe: conversion from 'int64' to 'unsigned int'
     pair<unsigned int, unsigned int> old_pair
-      (st->AddSymbol(old_symbol_pair.first),
-       st->AddSymbol(old_symbol_pair.second));
+      ((unsigned int)st->AddSymbol(old_symbol_pair.first),
+       (unsigned int)st->AddSymbol(old_symbol_pair.second));
     pair<unsigned int, unsigned int> new_pair
-      (st->AddSymbol(new_symbol_pair.first),
-       st->AddSymbol(new_symbol_pair.second));
+      ((unsigned int)st->AddSymbol(new_symbol_pair.first),
+       (unsigned int)st->AddSymbol(new_symbol_pair.second));
     StdVectorFst * retval = substitute(t, old_pair, new_pair);
     retval->SetInputSymbols(st);
     delete st;
@@ -1961,7 +2086,7 @@ namespace hfst {
     for( int i = 0; i < states; ++i ) {
 
       for (fst::MutableArcIterator<fst::StdVectorFst> it(t,i);
-           not it.Done();
+           ! it.Done();
            it.Next()) {
 
         fst::StdArc arc = it.Value();
@@ -2047,7 +2172,7 @@ namespace hfst {
     for( unsigned int i = 0; i < (unsigned int)states; ++i ) {
 
       for (fst::MutableArcIterator<fst::StdVectorFst> it(t,i);
-           not it.Done();
+           ! it.Done();
            it.Next()) {
 
         fst::StdArc arc = it.Value();
@@ -2188,7 +2313,7 @@ namespace hfst {
             }
           }
 
-        if (not transition_found) {
+        if (! transition_found) {
           StateId new_state = t->AddState();
           t->AddArc(s, StdArc(inumber,onumber,0,new_state));
           s = new_state;
@@ -2223,7 +2348,7 @@ namespace hfst {
             }
           }
 
-        if (not transition_found) {
+        if (! transition_found) {
           StateId new_state = t->AddState();
           t->AddArc(s, StdArc(inumber,onumber,0,new_state));
           s = new_state;
@@ -2237,6 +2362,9 @@ namespace hfst {
   StdVectorFst * TropicalWeightTransducer::intersect(StdVectorFst * t1,
                                                      StdVectorFst * t2)
   {
+    CHECK_EPSILON_CYCLES(t1, "intersect");
+    CHECK_EPSILON_CYCLES(t2, "intersect");
+
     RmEpsilon(t1);
     RmEpsilon(t2);
 
@@ -2277,6 +2405,9 @@ namespace hfst {
     if (t2->OutputSymbols() == NULL)
       t2->SetOutputSymbols(t2->InputSymbols());
 
+    CHECK_EPSILON_CYCLES(t1, "subtract");
+    CHECK_EPSILON_CYCLES(t2, "subtract");
+
     RmEpsilon(t1);
     RmEpsilon(t2);
 
@@ -2289,7 +2420,7 @@ namespace hfst {
     StdVectorFst *t2_ = copy(t2);
 
     for (fst::StateIterator<StdVectorFst> siter(*t2_); 
-         not siter.Done(); siter.Next())
+         ! siter.Done(); siter.Next())
       {
         StateId s = siter.Value();
         for (fst::MutableArcIterator<StdVectorFst> aiter(t2_,s); 
@@ -2340,7 +2471,7 @@ namespace hfst {
   {
     StdVectorFst * t_copy = new StdVectorFst(*t);
     for (fst::StateIterator<StdVectorFst> iter(*t); 
-         not iter.Done(); iter.Next())
+         ! iter.Done(); iter.Next())
       {
         if (t_copy->Final(iter.Value()) != fst::TropicalWeight::Zero())
           { t_copy->SetFinal(iter.Value(),f); }
@@ -2360,7 +2491,7 @@ namespace hfst {
          !aiter.Done();
          aiter.Next())
       {
-        if ((aiter.Value().ilabel == ilabel) and 
+        if ((aiter.Value().ilabel == ilabel) && 
             (aiter.Value().olabel == olabel))
           { return aiter.Position(); }
       }
@@ -2637,7 +2768,7 @@ namespace hfst {
 
       /* Go through all transitions in a random order.
      (If \a t is pruned, only one transition is proceeded.) */
-      while ( not t_transitions.empty() ) {
+      while ( ! t_transitions.empty() ) {
     unsigned int index = rand() % t_transitions.size();
     fst::StdArc arc = t_transitions.at(index);
     t_transitions.erase(t_transitions.begin()+index);
@@ -2854,7 +2985,7 @@ namespace hfst {
     }
 
     fst::SymbolTable *output_st=NULL;
-    if (not hfst_format) {
+    if (! hfst_format) {
       output_st = new fst::SymbolTable(*(transducer->InputSymbols()));
       transducer->SetOutputSymbols(output_st);
     }
