@@ -16,12 +16,15 @@
 #include "HfstExceptionDefs.h"
 #include "HfstFlagDiacritics.h"
 
-#ifdef WINDOWS
+#if HAVE_CONFIG_H
+  #include "../../../config.h"
+#endif
+
+#ifdef _MSC_VER
 #include "back-ends/openfstwin/src/include/fst/fstlib.h"
 #else
-#include "../../../config.h"
 #include "back-ends/openfst/src/include/fst/fstlib.h"
-#endif // WINDOWS
+#endif // _MSC_VER
 
 #include "HfstExtractStrings.h"
 #include <cstdio>
@@ -242,6 +245,9 @@ namespace implementations
 
       static FdTable<int64>* get_flag_diacritics(StdVectorFst * t);
 
+      static void add_to_weights(StdVectorFst * t, float w);
+      static float get_smallest_weight(StdVectorFst * t);
+
       static void print_alphabet(const StdVectorFst *t);
 
       // string versions
@@ -307,11 +313,16 @@ namespace implementations
         (StdVectorFst * t, 
          std::vector<std::pair<unsigned short, std::string> > symbol_mappings);
 
+      static void set_warning_stream(std::ostream * os);
+      static std::ostream * get_warning_stream();
+
     private:
       static fst::SymbolTable create_symbol_table(std::string name);
       static void initialize_symbol_tables(StdVectorFst *t);
       static void remove_symbol_table(StdVectorFst *t);      
 
+      static std::ostream * warning_stream;
+
       /* Maps state numbers in AT&T text format to state ids used by 
          OpenFst transducers. */
       typedef std::map<int, StateId> StateMap;
