@@ -161,6 +161,8 @@ void PmatchAlphabet::count(SymbolNumber sym)
 PmatchContainer::PmatchContainer(std::istream & inputstream):
     verbose(false),
     locate_mode(false),
+    profile_mode(false),
+    single_codepoint_tokenization(false),
     recursion_depth_left(PMATCH_MAX_RECURSION_DEPTH),
     entry_stack()
 {
@@ -393,9 +395,9 @@ std::string PmatchContainer::parse_name_from_hfst3_header(std::istream & f)
         if (headervalue[remaining_header_len - 1] != '\0') {
             HFST_THROW(TransducerHeaderException);
         }
-        char type[remaining_header_len];
+        char * type = new char [remaining_header_len];
         bool type_defined = false;
-        char name[remaining_header_len];
+        char * name = new char [remaining_header_len];
         bool name_defined = false;
         int i = 0;
         while (i < remaining_header_len) {
@@ -421,7 +423,10 @@ std::string PmatchContainer::parse_name_from_hfst3_header(std::istream & f)
         if (strcmp(type, "HFST_OLW") != 0) {
             HFST_THROW(TransducerHeaderException);
         }
-        return std::string(name);
+        std::string retval = std::string(name);
+        delete [] type;
+        delete [] name;
+        return retval;
     } else // nope. put back what we've taken
     {
         f.unget(); // first the non-matching character
@@ -481,6 +486,7 @@ void PmatchContainer::process(std::string & input_str)
     ++line_number;
     output.clear();
     locations.clear();
+    DoubleTape nonmatching_locations;
     while (has_queued_input(input_pos)) {
         SymbolNumber current_input = input[input_pos];
         if (not_possible_first_symbol(current_input)) {
@@ -488,15 +494,27 @@ void PmatchContainer::process(std::string & input_str)
             ++input_pos;
             if (locate_mode && alphabet.is_printable(current_input)) {
                 ++printable_input_pos;
+                nonmatching_locations.push_back(
+                    SymbolPair(current_input, current_input));
             }
             continue;
         }
         tape.clear();
         unsigned int tape_pos = 0;
+        unsigned int old_input_pos = input_pos;
         toplevel->match(input_pos, tape_pos);
         if (tape_pos > 0) {
             // Tape moved
             if (locate_mode) {
+                if (!nonmatching_locations.empty()) {
+                    LocationVector ls;
+                    Location nonmatching = alphabet.locatefy(printable_input_pos - nonmatching_locations.size(),
+                                                             WeightedDoubleTape(nonmatching_locations, 0.0));
+                    nonmatching.output = "@_NONMATCHING_@";
+                    ls.push_back(nonmatching);
+                    locations.push_back(ls);
+                    nonmatching_locations.clear();
+                }
                 LocationVector ls;
                 for (WeightedDoubleTapeVector::iterator it = (toplevel->locations)->begin();
                      it != (toplevel->locations)->end(); ++it) {
@@ -505,17 +523,29 @@ void PmatchContainer::process(std::string & input_str)
                 }
                 sort(ls.begin(), ls.end());
                 locations.push_back(ls);
+                printable_input_pos += (input_pos - old_input_pos);
             } else {
                 copy_to_output(toplevel->get_best_result());
             }
-        } else {
+        }
+        if (tape_pos == 0 || input_pos == old_input_pos) {
+            // If nothing happened, we move one position up
             copy_to_output(current_input, current_input);
             ++input_pos;
             if (locate_mode && alphabet.is_printable(current_input)) {
                 ++printable_input_pos;
+                nonmatching_locations.push_back(SymbolPair(current_input, current_input));
             }
         }
     }
+    if (locate_mode && !nonmatching_locations.empty()) {
+        LocationVector ls;
+        Location nonmatching = alphabet.locatefy(printable_input_pos - nonmatching_locations.size(),
+                                                 WeightedDoubleTape(nonmatching_locations, 0.0));
+        nonmatching.output = "@_NONMATCHING_@";
+        ls.push_back(nonmatching);
+        locations.push_back(ls);
+    }
 }
 
 std::string PmatchContainer::match(std::string & input)
@@ -922,28 +952,47 @@ void PmatchTransducer::collect_first(TransitionTableIndex i,
 
 void PmatchContainer::initialize_input(const char * input_s)
 {
+    input.clear();
     char * input_str = const_cast<char *>(input_s);
     char ** input_str_ptr = &input_str;
     SymbolNumber k = NO_SYMBOL_NUMBER;
     SymbolNumber boundary_sym = alphabet.get_special(boundary);
-    input.clear();
+    char * single_codepoint_scratch;
+    char * single_codepoint_scratch_orig;
+    if (single_codepoint_tokenization) {
+        single_codepoint_scratch = new char[5];
+        single_codepoint_scratch_orig = single_codepoint_scratch;
+    }
     if (boundary_sym != NO_SYMBOL_NUMBER) {
         input.push_back(boundary_sym);
     }
     while (**input_str_ptr != 0) {
         char * original_input_loc = *input_str_ptr;
-        k = encoder->find_key(input_str_ptr);
+        if (single_codepoint_tokenization) {
+            int bytes_to_tokenize = nByte_utf8(**input_str_ptr);
+            if (bytes_to_tokenize > 0) {
+                single_codepoint_scratch = single_codepoint_scratch_orig;
+                memcpy(single_codepoint_scratch, *input_str_ptr, bytes_to_tokenize);
+                single_codepoint_scratch[bytes_to_tokenize] = '\0';
+                k = encoder->find_key(&single_codepoint_scratch);
+                if (k != NO_SYMBOL_NUMBER) {
+                    (*input_str_ptr) += bytes_to_tokenize;
+                }
+            }
+        } else {
+            k = encoder->find_key(input_str_ptr);
+        }
         if (k == NO_SYMBOL_NUMBER) {
+            // Regular tokenization failed
             // the encoder moves as far as it can during tokenization,
             // we want to go back to be in position to add one utf-8 char
             *input_str_ptr = original_input_loc;
-            // Regular tokenization failed
             int bytes_to_tokenize = nByte_utf8(**input_str_ptr);
             if (bytes_to_tokenize == 0) {
-                // even if it's not utf-8, tokenize a byte at a time
+                // if utf-8 tokenization fails too, just grab a byte
                 bytes_to_tokenize = 1;
             }
-            char new_symbol[bytes_to_tokenize + 1];
+            char new_symbol[5];
             memcpy(new_symbol, *input_str_ptr, bytes_to_tokenize);
             new_symbol[bytes_to_tokenize] = '\0';
             (*input_str_ptr) += bytes_to_tokenize;
@@ -957,6 +1006,9 @@ void PmatchContainer::initialize_input(const char * input_s)
     if (boundary_sym != NO_SYMBOL_NUMBER) {
         input.push_back(boundary_sym);
     }
+    if (single_codepoint_tokenization) {
+        delete single_codepoint_scratch_orig;
+    }
     return;
 }
 
@@ -1139,6 +1191,7 @@ void PmatchTransducer::check_context(unsigned int input_pos,
                                      unsigned int tape_pos,
                                      TransitionTableIndex i)
 {
+//    std::cerr << local_stack.size() << std::endl;
     local_stack.top().context_placeholder = input_pos;
     if (local_stack.top().context == LC ||
         local_stack.top().context == NLC) {
@@ -1149,14 +1202,19 @@ void PmatchTransducer::check_context(unsigned int input_pos,
     // In case we have a negative context, we check to see if the context matched.
     // If it did, we schedule a passthrough arc after we've processed epsilons.
     bool schedule_passthrough = false;
+//            std::cerr << "!local_stack.top().negative_context_success is " <<
+//            !local_stack.top().negative_context_success << std::endl;
+
     if((local_stack.top().context == NLC || local_stack.top().context == NRC)
        && !local_stack.top().negative_context_success) {
         schedule_passthrough = true;
+//        std::cerr << "scheduled passthrough\n";
     }
     local_stack.pop();
     if (schedule_passthrough) {
         local_stack.top().pending_passthrough = true;
     }
+//    std::cerr << local_stack.size() << std::endl;
 }
 
 void PmatchTransducer::take_rtn(SymbolNumber input,
@@ -1231,13 +1289,18 @@ void PmatchTransducer::take_transitions(SymbolNumber input,
                 // input tape to find the symbol we want to write
                     this_output = container->input[input_pos];
                 }
+                if (this_input == alphabet.get_identity_symbol() ||
+                    (this_input == alphabet.get_unknown_symbol()) ||
+                    (alphabet.list2symbols[this_input] != NO_SYMBOL_NUMBER)) {
+                    this_input = container->input[input_pos];
+                }
                 Weight tmp = local_stack.top().running_weight;
                 local_stack.top().running_weight +=
                     transition_table[i].get_weight();
                 if (this_input == alphabet.get_special(Pmatch_passthrough)) {
                     get_analyses(input_pos, tape_pos, target); // FIXME
                 } else {
-                    container->tape.write(tape_pos, input, this_output);
+                    container->tape.write(tape_pos, this_input, this_output);
                     get_analyses(input_pos + 1, tape_pos + 1, target);
                 }
                 local_stack.top().running_weight = tmp;
@@ -1263,9 +1326,9 @@ void PmatchTransducer::get_analyses(unsigned int input_pos,
         }
         return;
     }
-
     local_stack.top().default_symbol_trap = true;
     take_epsilons(input_pos, tape_pos, i + 1);
+//    std::cerr << "get_analyses local stack size is " << local_stack.size() << std::endl;
     if (local_stack.top().pending_passthrough) {
         // A negative context failed
         take_transitions(alphabet.get_special(Pmatch_passthrough),
