@@ -14,6 +14,12 @@
 #ifndef _HFST_OL_TRANSDUCER_TRANSDUCER_H_
 #define _HFST_OL_TRANSDUCER_TRANSDUCER_H_
 
+#ifndef _MSC_VER
+#  include <unistd.h>
+#else
+#  include <io.h>
+#endif
+
 #include <vector>
 #include <set>
 #include <iostream>
@@ -26,11 +32,18 @@
 #include <deque>
 #include <queue>
 #include <stdexcept>
+#include <time.h>
 
 #include "../../HfstExceptionDefs.h"
 #include "../../HfstFlagDiacritics.h"
 #include "../../HfstSymbolDefs.h"
 
+#ifdef _MSC_VER
+ #include <BaseTsd.h>
+ typedef SSIZE_T ssize_t;
+#endif
+
+
 /** \brief A namespace for optimized-lookup functions and datatypes.*/
 namespace hfst_ol {
 using hfst::FdOperation;
@@ -75,10 +88,11 @@ struct TraversalState
 };
 typedef std::set<TraversalState> TraversalStates;
 
-const SymbolNumber NO_SYMBOL_NUMBER = std::numeric_limits<SymbolNumber>::max();
+  // parentheses avoid collision with windows macro 'max'
+  const SymbolNumber NO_SYMBOL_NUMBER = (std::numeric_limits<SymbolNumber>::max)();
 const TransitionTableIndex NO_TABLE_INDEX =
-    std::numeric_limits<TransitionTableIndex>::max();
-const unsigned long NO_COUNTER = std::numeric_limits<unsigned long>::max();
+  (std::numeric_limits<TransitionTableIndex>::max)();
+  const unsigned long NO_COUNTER = (std::numeric_limits<unsigned long>::max)();
 const Weight INFINITE_WEIGHT = static_cast<float>(NO_TABLE_INDEX);
 
 enum HeaderFlag {Weighted, Deterministic, Input_deterministic, Minimized,
@@ -458,8 +472,8 @@ public:
     void write(std::ostream& os, bool weighted) const
         {
             os.write(reinterpret_cast<const char*>(&input_symbol),
-                     sizeof(input_symbol));
-            if(!weighted and input_symbol == NO_SYMBOL_NUMBER and
+                     sizeof(SymbolNumber));
+            if(!weighted && input_symbol == NO_SYMBOL_NUMBER &&
                first_transition_index != NO_TABLE_INDEX) {
                 // Make sure that we write the correct type of final index
                 unsigned int unweighted_final_index = 1;
@@ -756,10 +770,17 @@ private:
     SymbolNumberVector symbols;
     
 public:
-    OlLetterTrie(void):
+    OlLetterTrie():
         letters(UCHAR_MAX, static_cast<OlLetterTrie*>(NULL)),
         symbols(UCHAR_MAX,NO_SYMBOL_NUMBER)
         {}
+
+    ~OlLetterTrie() {
+        for (size_t i=0 ; i<letters.size() ; ++i) {
+            delete letters[i];
+            letters[i] = 0;
+        }
+    }
     
     void add_string(const char * p,SymbolNumber symbol_key);
     bool has_key_starting_with(const char c) const;
@@ -835,6 +856,8 @@ protected:
 
     ssize_t max_lookups;
     unsigned int recursion_depth_left;
+    double max_time;
+    clock_t max_clock;
 
     void try_epsilon_transitions(unsigned int input_tape_pos,
                                  unsigned int output_tape_pos,
@@ -923,10 +946,10 @@ public:
     bool is_lookup_infinitely_ambiguous(const StringVector & s);
     bool is_lookup_infinitely_ambiguous(const std::string & input);
     
-    TransducerTable<TransitionWIndex> & copy_windex_table();
-    TransducerTable<TransitionW> & copy_transitionw_table();
-    TransducerTable<TransitionIndex> & copy_index_table();
-    TransducerTable<Transition> & copy_transition_table();
+    TransducerTable<TransitionWIndex> copy_windex_table();
+    TransducerTable<TransitionW> copy_transitionw_table();
+    TransducerTable<TransitionIndex> copy_index_table();
+    TransducerTable<Transition> copy_transition_table();
 
     // state_index must be an index to a state which is defined as either:
     // (1) the start of a set of entries in the transition index table, or
@@ -939,13 +962,16 @@ public:
 
 
     bool initialize_input(const char * input_str);
-    HfstOneLevelPaths * lookup_fd(const StringVector & s, ssize_t limit = -1);
+    HfstOneLevelPaths * lookup_fd(const StringVector & s, ssize_t limit = -1,
+        double time_cutoff = 0.0);
     /* Tokenize and lookup, accounting for flag diacritics, the surface string
        \a s. The return value, a pointer to HfstOneLevelPaths
        (which is a set) of analyses, is newly allocated.
     */
-    HfstOneLevelPaths * lookup_fd(const std::string & s, ssize_t limit = -1);
-    HfstOneLevelPaths * lookup_fd(const char * s, ssize_t limit = -1);
+    HfstOneLevelPaths * lookup_fd(const std::string & s, ssize_t limit = -1,
+                                  double time_cutoff = 0.0);
+    HfstOneLevelPaths * lookup_fd(const char * s, ssize_t limit = -1,
+                                  double time_cutoff = 0.0);
     void note_analysis(void);
 
     // Methods for supporting ospell
