@@ -1,9 +1,7 @@
 //! @file LexcCompiler.cc
 //!
 //! @brief Implementation of lexc compilation.
-//! 
-//! @author Tommi A. Pirinen
-//!
+
 
 //       This program is free software: you can redistribute it and/or modify
 //       it under the terms of the GNU General Public License as published by
@@ -41,7 +39,13 @@ using std::set_difference;
 #include "lexc-utils.h"
 #include "lexc-parser.hh"
 #include "xre_utils.h"
-#include "../../../tools/src/HfstStrings2FstTokenizer.h"
+#include "HfstSymbolDefs.h"
+//#include "../../../tools/src/HfstStrings2FstTokenizer.h"
+
+#ifdef WINDOWS
+#include "../../../tools/src/hfst-string-conversions.h"
+using hfst::hfst_fprintf_console;
+#endif // WINDOWS
 
 using hfst::HfstTransducer;
 using hfst::implementations::HfstTransitionGraph;
@@ -50,7 +54,7 @@ using hfst::implementations::HfstState;
 using hfst::implementations::HfstBasicTransition;
 using hfst::ImplementationType;
 using hfst::xre::XreCompiler;
-
+using hfst::StringVector;
 
 //using same alg as strings to fst
 static char *epsilonname=NULL; // FIX: use this
@@ -65,12 +69,14 @@ static StringVector multichar_symbols;
 extern FILE* hlexcin;
 extern int hlexcparse();
 extern int hlexcnerrs;
-extern void hlexclex_destroy();
+extern int hlexclex_destroy();
 
 #ifndef DEBUG_MAIN
 
 namespace hfst { namespace lexc {
 
+    bool debug = false;
+
 LexcCompiler* lexc_ = 0;
 
 LexcCompiler::LexcCompiler() :
@@ -84,11 +90,20 @@ LexcCompiler::LexcCompiler() :
     currentEntries_(0),
     parseErrors_(false),
     with_flags_(false),
+    align_strings_(false),
     minimize_flags_(false),
     rename_flags_(false),
-    allow_multiple_sublexicon_definitions_(false)
+    allow_multiple_sublexicon_definitions_(false),
+    error_(&std::cerr)
+#ifdef WINDOWS
+    , output_to_console_(false),
+    winoss_(std::ostringstream()),
+    redirected_stream_(NULL)
+#endif
 {
     xre_.set_expand_definitions(true);
+    xre_.set_error_stream(this->error_);
+    xre_.set_verbosity(!quiet_);
 }
 
 LexcCompiler::LexcCompiler(ImplementationType impl) :
@@ -102,9 +117,16 @@ LexcCompiler::LexcCompiler(ImplementationType impl) :
     currentEntries_(0),
     parseErrors_(false),
     with_flags_(false),
+    align_strings_(false),
     minimize_flags_(false),
     rename_flags_(false),
-    allow_multiple_sublexicon_definitions_(false)
+    allow_multiple_sublexicon_definitions_(false),
+    error_(&std::cerr)
+#ifdef WINDOWS
+    , output_to_console_(false),
+    winoss_(std::ostringstream()),
+    redirected_stream_(NULL)
+#endif
 {
     tokenizer_.add_multichar_symbol("@_EPSILON_SYMBOL_@");
     tokenizer_.add_multichar_symbol("@0@");
@@ -114,9 +136,11 @@ LexcCompiler::LexcCompiler(ImplementationType impl) :
     lexiconNames_.insert(hash);
     tokenizer_.add_multichar_symbol(joinerEncode(hash));
     xre_.set_expand_definitions(true);
+    xre_.set_error_stream(this->error_);
+    xre_.set_verbosity(!quiet_);
 }
 
-LexcCompiler::LexcCompiler(ImplementationType impl, bool withFlags) :
+LexcCompiler::LexcCompiler(ImplementationType impl, bool withFlags, bool alignStrings) :
     quiet_(false),
     verbose_(false),
     treat_warnings_as_errors_(false),
@@ -127,9 +151,16 @@ LexcCompiler::LexcCompiler(ImplementationType impl, bool withFlags) :
     currentEntries_(0),
     parseErrors_(false),
     with_flags_(withFlags),
+    align_strings_(alignStrings),
     minimize_flags_(false),
     rename_flags_(false),
-    allow_multiple_sublexicon_definitions_(false)
+    allow_multiple_sublexicon_definitions_(false),
+    error_(&std::cerr)
+#ifdef WINDOWS
+    , output_to_console_(false),
+    winoss_(std::ostringstream()),
+    redirected_stream_(NULL)
+#endif
 {
     tokenizer_.add_multichar_symbol("@_EPSILON_SYMBOL_@");
     tokenizer_.add_multichar_symbol("@0@");
@@ -139,9 +170,59 @@ LexcCompiler::LexcCompiler(ImplementationType impl, bool withFlags) :
     lexiconNames_.insert(hash);
     tokenizer_.add_multichar_symbol(joinerEncode(hash));
     xre_.set_expand_definitions(true);
+    xre_.set_error_stream(this->error_);
+    xre_.set_verbosity(!quiet_);
 }
 
 
+    std::ostream * LexcCompiler::get_stream(std::ostream * oss)
+    {
+#ifdef WINDOWS
+      if (output_to_console_ && (oss == &std::cerr || oss == &std::cout))
+        {
+          redirected_stream_ = oss;
+          return &winoss_;
+        }
+#endif
+      return oss;
+    }
+
+    void LexcCompiler::flush(std::ostream * oss)
+    {
+#ifdef WINDOWS
+      if (output_to_console_ && (oss == &winoss_))
+        {
+          if (redirected_stream_ == &std::cerr)
+            hfst_fprintf_console(stderr, winoss_.str().c_str());
+          else if (redirected_stream_ == &std::cout)
+            hfst_fprintf_console(stdout, winoss_.str().c_str());
+          else
+            ;
+          redirected_stream_ = NULL;
+          winoss_.str("");
+        }
+#endif
+    }
+
+    void LexcCompiler::setOutputToConsole(bool value)
+    {
+#ifdef WINDOWS
+      output_to_console_ = value;
+#else
+      (void)value;
+#endif
+    }
+
+    bool LexcCompiler::getOutputToConsole()
+    {
+#ifdef WINDOWS
+      return output_to_console_;
+#else
+      return false;
+#endif
+    }
+
+
 LexcCompiler& LexcCompiler::parse(FILE* infile)
 {
     lexc_ = this;
@@ -171,7 +252,10 @@ LexcCompiler& LexcCompiler::parse(const char* filename)
     hlexcin = fopen(filename, "r");
     if (hlexcin == NULL)
       {
-        fprintf(stderr, "could not open %s for reading\n", filename);
+        std::ostream * err = get_stream(error_);
+        *err << "could not open " << filename << " for reading" << std::endl;
+        flush(err);  
+        //fprintf(stderr, "could not open %s for reading\n", filename);
         parseErrors_ = true;
         return *this;
       }
@@ -188,6 +272,17 @@ bool LexcCompiler::isQuiet()
   return quiet_;
 }
 
+unsigned int LexcCompiler::getVerbosity()
+{
+  if (quiet_ && !verbose_)
+    return 0;
+  if (!quiet_ && !verbose_)
+    return 1;
+  if (!quiet_ && verbose_)
+    return 2;
+  throw "LexcCompiler::getVerbosity() failed";
+}
+
 LexcCompiler&
 LexcCompiler::setVerbosity(unsigned int verbose)
 {
@@ -208,6 +303,7 @@ LexcCompiler::setVerbosity(unsigned int verbose)
       quiet_ = false;
       verbose_ = true;
     }
+  xre_.set_verbosity(!quiet_);
   return *this;
 }
 
@@ -224,6 +320,17 @@ LexcCompiler::setTreatWarningsAsErrors(bool value)
     return *this;
 }
 
+void LexcCompiler::set_error_stream(std::ostream * os)
+{
+  error_ = os;
+  xre_.set_error_stream(this->error_);
+}
+
+std::ostream * LexcCompiler::get_error_stream()
+{
+  return error_;
+}
+
 LexcCompiler&
 LexcCompiler::setAllowMultipleSublexiconDefinitions(bool value)
 {
@@ -231,6 +338,12 @@ LexcCompiler::setAllowMultipleSublexiconDefinitions(bool value)
     return *this;
 }
 
+LexcCompiler&
+LexcCompiler::setAlignStrings(bool value)
+{
+    align_strings_ = value;
+    return *this;
+}
 
 LexcCompiler&
 LexcCompiler::setWithFlags(bool value)
@@ -324,6 +437,7 @@ LexcCompiler::addStringEntry(const string& data,
 // to handle information to warn_about_one_sided_flags_
 static bool treat_one_sided_flags_as_errors_ = false;
 static bool quiet_one_sided_flags_ = false;
+static std::ostream * errorstr_ = NULL;
 
 static void warn_about_one_sided_flags(const std::pair<std::string, std::string> & symbol_pair)
 {
@@ -335,7 +449,8 @@ static void warn_about_one_sided_flags(const std::pair<std::string, std::string>
             {
               if (true /*!quiet_one_sided_flags_*/) // error messages are always printed
                 {
-                  std::cerr << std::endl << "*** ERROR: one-sided flag diacritic: " << symbol_pair.first << ":" << symbol_pair.second << " [--Werror]" << std::endl;
+                  *errorstr_ << std::endl << "*** ERROR: one-sided flag diacritic: " << symbol_pair.first << ":" << symbol_pair.second << " [--Werror]" << std::endl;
+                  lexc_->flush(errorstr_);
                 }
               throw "one-sided flag";
             }
@@ -352,7 +467,8 @@ static void warn_about_one_sided_flags(const std::pair<std::string, std::string>
         {
           if (true /*!quiet_one_sided_flags_*/) // error messages are always printed
             {
-              std::cerr << std::endl << "*** ERROR: one-sided flag diacritic: " << symbol_pair.first << ":" << symbol_pair.second << " [--Werror]" << std::endl;
+              *hfst::lexc::errorstr_ << std::endl << "*** ERROR: one-sided flag diacritic: " << symbol_pair.first << ":" << symbol_pair.second << " [--Werror]" << std::endl;
+              lexc_->flush(errorstr_);
             }
           throw "one-sided flag";
         }
@@ -412,61 +528,105 @@ LexcCompiler::addStringPairEntry(const string& upper, const string& lower,
     }
     tokenizer_.add_multichar_symbol(joinerEnc);
 
-    StringPairVector upperV;
-    upperV = tokenizer_.tokenize(upper_string);
-
-    StringPairVector lowerV;
-    lowerV = tokenizer_.tokenize(lower_string);
-
-    int upperSize = upperV.size();
-    int lowerSize = lowerV.size();
-
-    //cout << "u: "  << upperSize << "\n";
-    //cout << "L: "  << lowerSize << "\n";
-
-
-
-    StringVector upV;
-    StringVector loV;
-
-    StringPairVector newVector;
-
     // information for function pointer &warn_about_one_sided_flags
     treat_one_sided_flags_as_errors_ = treat_warnings_as_errors_;
     quiet_one_sided_flags_ = quiet_;
+    errorstr_ = this->get_stream(this->error_);
+    
+    StringPairVector newVector;
 
-    if ( upperSize > lowerSize)
+    
+    if (align_strings_)
     {
-        std::string epsilons = "";
-        for(int i=1; i <= upperSize-lowerSize ; i++)
+        StringPairVector tmp = tokenizer_.tokenize(upper_string, lower_string);
+        vector<string> one;
+        vector<string> two;
+        
+
+        for(StringPairVector::iterator it = tmp.begin() ; it < tmp.end(); ++it)
         {
-            //cout << "another epsilon \n";
-            epsilons = epsilons + string("@@ANOTHER_EPSILON@@");
+            if (it->first != "@_EPSILON_SYMBOL_@" )
+                one.push_back(it->first);
+            if (it->second != "@_EPSILON_SYMBOL_@" )
+                two.push_back(it->second);
+        }
+        
+        
+        pair<vector<string>, vector<string> > med_vectors = find_med_alingment(one, two);
+        
+        
+        std::stringstream ss1;
+        for(size_t i = 0; i < med_vectors.first.size(); ++i)
+        {
+          ss1 << med_vectors.first[i];
+        }
+        std::string as1 = ss1.str();
+        
+         std::stringstream ss2;
+        for(size_t i = 0; i < med_vectors.second.size(); ++i)
+        {
+          ss2 << med_vectors.second[i];
+        }
+        std::string as2 = ss2.str();
+        
+        
+        newVector = tokenizer_.tokenize(joinerEnc + as1 + encodedCont,
+                                            joinerEnc + as2 + encodedCont,
+                                            &warn_about_one_sided_flags);
+    }else
+    {
+        StringPairVector upperV;
+        upperV = tokenizer_.tokenize(upper_string);
+
+        StringPairVector lowerV;
+        lowerV = tokenizer_.tokenize(lower_string);
+
+        int upperSize = upperV.size();
+        int lowerSize = lowerV.size();
+
+        //cout << "u: "  << upperSize << "\n";
+        //cout << "L: "  << lowerSize << "\n";
+
+
+
+        // StringVector upV;
+        // StringVector loV;
+        
+
+        if ( upperSize > lowerSize)
+        {
+            std::string epsilons = "";
+            for(int i=1; i <= upperSize-lowerSize ; i++)
+            {
+                //cout << "another epsilon \n";
+                epsilons = epsilons + string("@@ANOTHER_EPSILON@@");
+
+            }
+            newVector = tokenizer_.tokenize(joinerEnc + upper_string + encodedCont,
+                                            joinerEnc + lower_string + epsilons + encodedCont,
+                                            &warn_about_one_sided_flags);
 
         }
-        newVector = tokenizer_.tokenize(joinerEnc + upper_string + encodedCont,
-                                        joinerEnc + lower_string + epsilons + encodedCont,
-                                        &warn_about_one_sided_flags);
-
-    }
-    else if (upperSize < lowerSize)
-    {
-        std::string epsilons = "";
-        for(int i=1; i <= lowerSize-upperSize ; i++)
+        else if (upperSize < lowerSize)
         {
-            //cout << "another epsilon \n";
-            epsilons = epsilons + string("@@ANOTHER_EPSILON@@");
+            std::string epsilons = "";
+            for(int i=1; i <= lowerSize-upperSize ; i++)
+            {
+                //cout << "another epsilon \n";
+                epsilons = epsilons + string("@@ANOTHER_EPSILON@@");
 
+            }
+            newVector = tokenizer_.tokenize(joinerEnc + upper_string + epsilons + encodedCont,
+                                            joinerEnc + lower_string + encodedCont,
+                                            &warn_about_one_sided_flags);
         }
-        newVector = tokenizer_.tokenize(joinerEnc + upper_string + epsilons + encodedCont,
-                                        joinerEnc + lower_string + encodedCont,
-                                        &warn_about_one_sided_flags);
-    }
-    else
-    {
-        newVector = tokenizer_.tokenize(joinerEnc + upper_string + encodedCont,
-                                        joinerEnc + lower_string + encodedCont,
-                                        &warn_about_one_sided_flags);
+        else
+        {
+            newVector = tokenizer_.tokenize(joinerEnc + upper_string + encodedCont,
+                                            joinerEnc + lower_string + encodedCont,
+                                            &warn_about_one_sided_flags);
+        }
+        
     }
     stringsTrie_.disjunct(newVector, weight);
 
@@ -539,8 +699,12 @@ LexcCompiler::addXreEntry(const string& regexp, const string& continuation,
       {
         if ((currentEntries_ % 10000) == 0)
           {
-            fprintf(stderr, SIZE_T_SPECIFIER "...", currentEntries_);
+            std::ostream * err = get_stream(error_);
+            *err << currentEntries_ << "...";
+            flush(err);
+            //fprintf(stderr, SIZE_T_SPECIFIER "...", currentEntries_);
           }
+          
       }
 
 
@@ -578,9 +742,12 @@ LexcCompiler::addXreDefinition(const string& definition_name, const string& xre)
     xre_.define(definition_name, xre);
     if (!quiet_)
       {
-        fprintf(stderr,
-            "Defined '%s': ? Kb., ? states, ? arcs, ? paths.\n",
-            definition_name.c_str());
+        std::ostream * err = get_stream(error_);
+        *err << "Defined '" << definition_name << "': ? Kb., ? states, ? arcs, ? paths." << std::endl;
+        flush(err);
+        //fprintf(stderr,
+        //    "Defined '%s': ? Kb., ? states, ? arcs, ? paths.\n",
+        //    definition_name.c_str());
       }
     return *this;
 }
@@ -613,6 +780,7 @@ LexcCompiler::setCurrentLexiconName(const string& lexiconName)
         tokenizer_.add_multichar_symbol(encodedName);
     }
 
+    std::ostream * err = get_stream(error_);
 
     if ((firstLexicon) && (lexiconName == "Root"))
     {
@@ -620,21 +788,24 @@ LexcCompiler::setCurrentLexiconName(const string& lexiconName)
     }
     else if ((firstLexicon) && (lexiconName != "Root"))
     {
-        if (!quiet_) fprintf(stderr, "first lexicon is not named Root\n");
+      if (!quiet_) *err << "first lexicon is not named Root" << std::endl; // fprintf(stderr, "first lexicon is not named Root\n");
         setInitialLexiconName(lexiconName);
     }
     else if ((!firstLexicon) && (lexiconName == "Root"))
     {
-        if (!quiet_) fprintf(stderr, "Root is not first the first lexicon\n");
+      if (!quiet_) *err << "Root is not first the first lexicon" << std::endl; // fprintf(stderr, "Root is not first the first lexicon\n");
         setInitialLexiconName(lexiconName);
     }
     if (!firstLexicon && !quiet_)
     {
-        fprintf(stderr, SIZE_T_SPECIFIER " ", currentEntries_);
+      *err << currentEntries_ << " ";
+      // fprintf(stderr, SIZE_T_SPECIFIER " ", currentEntries_);
     }
-    if (!quiet_) fprintf(stderr, "%s...", lexiconName.c_str());
+    if (!quiet_) *err << lexiconName << "..."; // fprintf(stderr, "%s...", lexiconName.c_str());
     firstLexicon = false;
 
+    flush(err);
+
     currentEntries_ = 0;
     return *this;
 }
@@ -661,11 +832,14 @@ LexcCompiler::compileLexical()
         return 0;
       }
 
+    std::ostream * err = get_stream(error_);
+
     bool warnings_generated = false;
     printConnectedness(warnings_generated);
     if (warnings_generated && treat_warnings_as_errors_)
       {
-        if (!quiet_) fprintf(stderr, "*** ERROR: could not parse lexc file: treating warnings as errors [--Werror] ***\n");
+        if (!quiet_) *err << "*** ERROR: could not parse lexc file: treating warnings as errors [--Werror] ***" << std::endl; //fprintf(stderr, "*** ERROR: could not parse lexc file: treating warnings as errors [--Werror] ***\n");
+        flush(err);
         return 0;
       }
 
@@ -724,7 +898,9 @@ LexcCompiler::compileLexical()
         {
             if (verbose_)
             {
-                fprintf(stderr, "Morphotaxing... %s ", s->c_str());
+              *err << "Morphotaxing... " << *s << " ";
+              flush(err);
+              //  fprintf(stderr, "Morphotaxing... %s ", s->c_str());
             }
             string joinerEnc = *s;
             joinerEncode(joinerEnc);
@@ -806,7 +982,9 @@ LexcCompiler::compileLexical()
           {
             if (verbose_)
             {
-            fprintf(stderr, "Morphotaxing... %s ", s->c_str());
+              *err << "Morphotaxing... " << *s << " ";
+              flush(err);
+              //fprintf(stderr, "Morphotaxing... %s ", s->c_str());
             }
             string flagPstring = *s;
             string flagRstring = *s;
@@ -871,24 +1049,39 @@ LexcCompiler::compileLexical()
         joinersAll.repeat_star();
         joinersAll.minimize();
 
-/*
-     printf("lexicons before compose: \n");
-     lexicons.write_in_att_format(stdout, 1);
-
-      printf("joinersAll: \n");
-      joinersAll.write_in_att_format(stdout, 1);
-      printf("\n");
-*/
+        if (debug) 
+          {
+            *err << "lexicons before compose: " << std::endl;
+            *err << lexicons;
+            //fprintf(stderr, "lexicons before compose: \n");
+            //lexicons.write_in_att_format(stderr, 1);
+            
+            *err << "joinersAll: " << std::endl;
+            *err << joinersAll;
+            *err << std::endl;
+            //fprintf(stderr, "joinersAll: \n");
+            //joinersAll.write_in_att_format(stderr, 1);
+            //fprintf(stderr, "\n");
+            flush(err);
+          }
 
         lexicons.compose(joinersAll).minimize();
 
+        if (debug) 
+          {
+            *err << "lexicons after composition: " << std::endl;
+            *err << lexicons << std::endl; 
+            flush(err);
+          }
 
         HfstSymbolSubstitutions allSubstitutions;
         if(with_flags_)
         {
             if (verbose_)
             {
-                fprintf(stderr, "\nChanging flags...\n");
+              *err << std::endl << "Changing flags..." << std::endl;
+              flush(err);
+              //  fprintf(stderr, "\nChanging flags...\n");
             }
             HfstSymbolSubstitutions fakeFlagsToRealFlags;
             // Change fake flags to real flags
@@ -899,7 +1092,12 @@ LexcCompiler::compileLexical()
                            s != transducerAlphabet.end();
                            ++s)
             {
-                //printf("%s \n", s->c_str());
+              if (debug) 
+                { 
+                  *err << "handling alpha: '" << *s << "'..." << std::endl;
+                  flush(err);
+                  //fprintf(stderr, "handling alpha: '%s'...\n", s->c_str()); 
+                }
                 String alph = *s;
 
                 if ( alph[0] == '$' && *alph.rbegin() == '$' && alph.size() > 2)
@@ -907,6 +1105,11 @@ LexcCompiler::compileLexical()
                     replace(alph.begin(), alph.end(), '$', '@');
                     //std::cout << alph << '\n';
                     fakeFlagsToRealFlags.insert(StringPair(*s, alph));
+                    if (debug) 
+                      {
+                        *err << "debug: inserting fakeFlagsToRealFlags replacement: " << *s << " -> " << alph << std::endl; 
+                        flush(err);
+                      }
                     //lexicons.substitute(*s, alph).minimize();
                 }
             }
@@ -925,12 +1128,19 @@ LexcCompiler::compileLexical()
         lexicons.substitute(allSubstitutions).minimize();
         lexicons.prune_alphabet();
 
-
-
+        if (debug) 
+          {
+            *err << "lexicons after substitution: " << std::endl;
+            *err << lexicons << std::endl; 
+            flush(err);
+          }
+        
         //replace reg exp key with transducers
         if (verbose_)
         {
-            fprintf(stderr, "\nInserting regular expressions...\n");
+          *err << std::endl << "Inserting regular expressions..." << std::endl;
+          flush(err);
+          //fprintf(stderr, "\nInserting regular expressions...\n");
         }
 
 
@@ -948,6 +1158,11 @@ LexcCompiler::compileLexical()
 
                 //std::cout << alph << '\n';
                 fakeRegexprToReal.insert(StringPair(it->first, alph));
+                if (debug) 
+                  {
+                    *err << "debug: inserting fakeRegexprToReal replacement: " << it->first << " -> " << alph << std::endl; 
+                    flush(err);
+                  }
 
             //    lexicons.substitute(it->first, alph).minimize();
             }
@@ -979,17 +1194,28 @@ LexcCompiler::compileLexical()
             }
             HfstBasicTransducer btr(*(it->second));
             regMarkToTr[alph] = btr;
+            if (debug) 
+              { 
+                *err << "debug: regMarkToTr[" << alph << "] = " << std::endl;
+                btr.write_in_att_format(*err);
+                flush(err);
+              }
             //lexicons.substitute(StringPair(alph, alph), *it->second, true).minimize();
         }
       
-
         HfstBasicTransducer lexicons_basic(lexicons);
         lexicons_basic.substitute(regMarkToTr, true);
 
-
-
         lexicons_basic.prune_alphabet();
 
+        if (debug) 
+          {
+            *err << "lexicons_basic after regexp substitution: " << std::endl;
+            lexicons_basic.write_in_att_format(*err);
+            flush(err);
+          }
+
+
 /*
         printf("lexicons after substitute: \n");
         lexicons_basic.write_in_att_format(stdout, 1);
@@ -1002,125 +1228,79 @@ LexcCompiler::compileLexical()
 
     // Preserve only first flag of consecutive P and R lexname flag series, 
     // e.g. change P.LEXNAME.1 R.LEXNAME.1 P.LEXNAME.2 R.LEXNAME.2 into P.LEXNAME.1 
-        
-    if (with_flags_ && minimize_flags_)
-          {
-
-            // To substitute "@[P|R].LEXNAME...@" with "$[P|R].LEXNAME...$" and vice versa. 
-            std::map<String, String> flag_substitutions;             // @ -> $
-            std::map<String, String> reverse_flag_substitutions;     // $ -> @
-
-            // TEST: rename all $P$ and $R$ flags so that they have the same name
-            std::map<String, String> dollar_to_foo_substitutions;
-
-            StringSet transducerAlphabet = rv->get_alphabet();
-            for (StringSet::const_iterator s = transducerAlphabet.begin();
-                 s != transducerAlphabet.end();
-                 ++s)
+    if (with_flags_) //&& minimize_flags_)
+    {
+        StringSet transducerAlphabet = rv->get_alphabet();
+        StringSet flagD;
+        for (StringSet::const_iterator s = transducerAlphabet.begin();
+             s != transducerAlphabet.end();
+             ++s)
+        {
+            String alph = *s;
+            String alph10 = alph.substr(0,10);
+            if ( alph10 == "@P.LEXNAME" || alph10 == "@R.LEXNAME" )  
             {
-              String alph = *s;
-              String alph10 = alph.substr(0,10);
-              if ( alph10 == "@P.LEXNAME" || alph10 == "@R.LEXNAME" )  
-                {
-                  replace(alph.begin(), alph.end(), '@', '$');
-                  //fprintf(stderr, "flag found:\t\t%s\t->\t%s\n", s->c_str(), alph.c_str());
-                  flag_substitutions[*s] = alph;
-                  reverse_flag_substitutions[alph] = *s;
-                  // insert mapping: $[P|R].LEXNAME.SOMETHING$  <->  $[P|R]$.FOO.BAR$
-                  dollar_to_foo_substitutions[alph] = alph.substr(0, 2).append(".FOO.BAR$");
-                  //fprintf(stderr, "dollar_to_foo_substitutions[%s] = %s\n", alph.c_str(), (dollar_to_foo_substitutions[alph]).c_str());
-                }
-              else
-                {
-                  //fprintf(stderr, "other found:\t\t%s\n", alph.c_str());
-                }
+                flagD.insert(alph);
             }
+        }
 
-            // Substitute "@[P|R].LEXNAME...@" with "$[P|R].LEXNAME...$"
-            rv->substitute(flag_substitutions);
+        // Construct a rule for consecutive flag removal: 
+        // [FLAG1 | FLAG2 ... FLAGN] -> 0 || [FLAG1 | FLAG2 ... FLAGN] _
+        // and also an inverted rule
+        std::string flag_remover_regexp("[ ");
+        bool first_flag = true;
+        
+        for (StringSet::const_iterator it 
+               = flagD.begin(); it != flagD.end(); ++it)
+        {
+            if (!first_flag)
+            {
+                flag_remover_regexp.append("| ");
+            }
+            flag_remover_regexp.append("\"").append(*it).append("\" ");
+            first_flag = false;
+        }
+        flag_remover_regexp.append("]");
+        std::string context_regexp(flag_remover_regexp);
+        flag_remover_regexp.append(" -> 0 || ").append(context_regexp).append(" _ ");
 
-            //fprintf(stderr, "rv:\n");
-            //rv->write_in_att_format(stderr, 1);
-            //fprintf(stderr, "--\n");
+        if (debug) 
+        { 
+            fprintf(stderr, "flag_remover_regexp: %s\n", flag_remover_regexp.c_str()); 
+        }
+         
+        hfst::xre::XreCompiler xre_comp(format_);
+        
+        HfstTransducer * flag_filter = xre_comp.compile(flag_remover_regexp);
+        flag_filter->minimize();
+        HfstTransducer * inverted_flag_filter = new HfstTransducer(*flag_filter);
+        inverted_flag_filter->invert().minimize();
 
-            
-            // Construct a rule for consecutive flag removal: 
-            // [FLAG1 | FLAG2 ... FLAGN] -> 0 || [FLAG1 | FLAG2 ... FLAGN] _
-            // and also an inverted rule
-            std::string flag_remover_regexp("[ ");
-            bool first_flag = true;
-            for (std::map<String, String>::const_iterator it 
-                   = flag_substitutions.begin(); it != flag_substitutions.end(); it++)
-              {
-                //fprintf(stderr, "adding flag to rule:\t%s\n", it->second.c_str());
-                if (!first_flag)
-                      {
-                        flag_remover_regexp.append("| ");
-                      }
-                flag_remover_regexp.append("\"").append(it->second).append("\" ");
-                first_flag = false;
-              }
-            flag_remover_regexp.append("]");
-            std::string context_regexp(flag_remover_regexp);
-            flag_remover_regexp.append(" -> 0 || ").append(context_regexp).append(" _ ");
-            
-            // DEBUG
-            //fprintf(stderr, "flag_remover_regexp: %s\n", flag_remover_regexp.c_str());
-            
-            hfst::xre::XreCompiler xre_comp(format_);
-            HfstTransducer * flag_filter = xre_comp.compile(flag_remover_regexp);
-            flag_filter->minimize();
-            HfstTransducer * inverted_flag_filter = new HfstTransducer(*flag_filter);
-            inverted_flag_filter->invert().minimize();
-            
-            //fprintf(stderr, "flag_filter:\n");
-            //flag_filter->write_in_att_format(stderr, 1);
-            //fprintf(stderr, "--\n");            
-
-            //fprintf(stderr, "inverted flag_filter:\n");
-            //inverted_flag_filter->write_in_att_format(stderr, 1);
-            //fprintf(stderr, "--\n");            
-            
-
-            // [ [FLAG1 | FLAG2 ... FLAGN] -> 0 || [FLAG1 | FLAG2 ... FLAGN] _ ].inv
-            //                        .o.
-            //                       RESULT
-            //                        .o. 
-            // [FLAG1 | FLAG2 ... FLAGN] -> 0 || [FLAG1 | FLAG2 ... FLAGN] _
-            HfstTransducer filtered_lexicons(*inverted_flag_filter);
-            filtered_lexicons.harmonize_flag_diacritics(*rv);
-            filtered_lexicons.compose(*rv, true);
-            filtered_lexicons.harmonize_flag_diacritics(*flag_filter);
-            filtered_lexicons.compose(*flag_filter, true).minimize();
-
-            //fprintf(stderr, "filtered_lexicons:\n");
-            //filtered_lexicons.write_in_att_format(stderr, 1);
-            //fprintf(stderr, "--\n");                        
-
-            // TEST: Rename all $P$ and $R$ flags so that they have the same name
-            if (rename_flags_)
-              {
-                filtered_lexicons.substitute(dollar_to_foo_substitutions);
-                filtered_lexicons.substitute("$P.FOO.BAR$", "@P.FOO.BAR@");
-                filtered_lexicons.substitute("$R.FOO.BAR$", "@R.FOO.BAR@");
-              }
-            else
-              {
-                // Convert symbols "$[P|R].LEXNAME...$" back to "@[P|R].LEXNAME...@"            
-                filtered_lexicons.substitute(reverse_flag_substitutions);
-              }
-
-            rv->assign(filtered_lexicons);
-          }
+        // [ [FLAG1 | FLAG2 ... FLAGN] -> 0 || [FLAG1 | FLAG2 ... FLAGN] _ ].inv
+        //                        .o.
+        //                       RESULT
+        //                        .o. 
+        // [FLAG1 | FLAG2 ... FLAGN] -> 0 || [FLAG1 | FLAG2 ... FLAGN] _
+        HfstTransducer filtered_lexicons(*inverted_flag_filter);
+        filtered_lexicons.compose(*rv, true);
+        filtered_lexicons.compose(*flag_filter, true).minimize();
+         
+        rv->assign(filtered_lexicons);
+    }
 
     rv->minimize();
+    
+    if(!quiet_) *err << endl;
+    
     return rv;
 }
 
 
 const LexcCompiler&
-LexcCompiler::printConnectedness(bool & warnings_generated) const
+LexcCompiler::printConnectedness(bool & warnings_generated)
 {
+  std::ostream * err = get_stream(error_);
+
   if (/* !quiet_ && */ (lexiconNames_ != continuations_))
     {
         vector<string> lexMinusCont = vector<string>(lexiconNames_.size());
@@ -1140,9 +1320,11 @@ LexcCompiler::printConnectedness(bool & warnings_generated) const
             {
               if (!quiet_)
                 {
-                  fprintf(stderr,
-                          "Warning: Sublexicon is mentioned but not defined."
-                          " (%s) \n", s->c_str());
+                  *err << "Warning: Sublexicon is mentioned but not defined. (" << *s << ") " << std::endl;
+                  flush(err);
+                  //fprintf(stderr,
+                  //        "Warning: Sublexicon is mentioned but not defined."
+                  //        " (%s) \n", s->c_str());
                 }
               warnings_generated = true;
             }
@@ -1152,13 +1334,17 @@ LexcCompiler::printConnectedness(bool & warnings_generated) const
           warnings_generated = true;
           if (!quiet_)
             {
-              fprintf(stderr, "Warning: Sublexicons defined but not used\n");
+              *err << "Warning: Sublexicons defined but not used" << std::endl;
+              //fprintf(stderr, "Warning: Sublexicons defined but not used\n");
               for (vector<string>::iterator s = lexMinusCont.begin();
                    s != lexMinusContEnd; ++s)
                 {
-                  fprintf(stderr, "%s ", s->c_str());
+                  *err << *s << " ";
+                  //fprintf(stderr, "%s ", s->c_str());
                 }
-              fprintf(stderr, "\n");
+              *err << std::endl;
+              //fprintf(stderr, "\n");
+              flush(err);
             }
         }
     }
