@@ -11,7 +11,7 @@
 
 #include "pmatch_utils.h"
 #include "HfstTransducer.h"
-#include "tools/src/HfstUtf8.h"
+//#include "tools/src/HfstUtf8.h"
 #include "implementations/optimized-lookup/pmatch.h"
 
 using std::string;
@@ -45,6 +45,13 @@ pmatcherror(const char *msg)
 
 }
 
+void pmatchwarning(const char *msg)
+{
+    if (hfst::pmatch::verbose) {
+        std::cerr << "pmatch: "<< msg << std::endl;
+    }
+}
+
 namespace hfst 
 { 
 namespace pmatch 
@@ -196,6 +203,17 @@ add_percents(const char *s)
     return ns;
   }
 
+char*
+strip_newline(char *s)
+{
+  for (unsigned int pos = 0; s[pos] != '\0'; pos++)
+    {
+      if (s[pos] == '\n' || s[pos] == '\r')
+        s[pos] = '\0';
+    }
+  return s;
+}
+
 char *
 get_Ins_transition(const char *s)
 {
@@ -696,30 +714,35 @@ compile(const string& pmatch, map<string,HfstTransducer*>& defs,
         std::cerr << "\nHarmonizing... ";
     }
 
-    HfstTransducer dummy(format);
-    // We keep TOP and any inserted transducers
-    std::map<std::string, hfst::HfstTransducer *>::iterator defs_itr;
-    for (defs_itr = definitions.begin(); defs_itr != definitions.end();
-         ++defs_itr) {
-        if (defs_itr->first.compare("TOP") == 0 ||
-            inserted_transducers.count(defs_itr->first) != 0) {
-            dummy.harmonize(*defs_itr->second);
+    if (inserted_transducers.size() > 0) {
+
+        HfstTransducer dummy(format);
+        // We keep TOP and any inserted transducers
+        std::map<std::string, hfst::HfstTransducer *>::iterator defs_itr;
+        for (defs_itr = definitions.begin(); defs_itr != definitions.end();
+             ++defs_itr) {
+            if (defs_itr->first.compare("TOP") == 0 ||
+                inserted_transducers.count(defs_itr->first) != 0) {
+                dummy.harmonize(*defs_itr->second);
+            }
         }
-    }
-    
-    // Now that dummy is harmonized with everything, we harmonize everything
-    // with dummy and insert them into the result
-    for(defs_itr = definitions.begin(); defs_itr != definitions.end();
-        ++defs_itr) {
-        if (defs_itr->first.compare("TOP") == 0 ||
-            inserted_transducers.count(defs_itr->first) != 0) {
-            dummy.harmonize(*defs_itr->second);
-            retval.insert(std::pair<std::string, hfst::HfstTransducer*>(
-                              defs_itr->first,
-                              defs_itr->second));
-        } else {
-            delete defs_itr->second;
+        
+        // Now that dummy is harmonized with everything, we harmonize everything
+        // with dummy and insert them into the result
+        for(defs_itr = definitions.begin(); defs_itr != definitions.end();
+            ++defs_itr) {
+            if (defs_itr->first.compare("TOP") == 0 ||
+                inserted_transducers.count(defs_itr->first) != 0) {
+                dummy.harmonize(*defs_itr->second);
+                retval.insert(std::pair<std::string, hfst::HfstTransducer*>(
+                                  defs_itr->first,
+                                  defs_itr->second));
+            } else {
+                delete defs_itr->second;
+            }
         }
+    } else {
+        retval.insert(std::pair<std::string, hfst::HfstTransducer*>("TOP", definitions["TOP"]));
     }
     if (hfst::pmatch::verbose) {
         double duration = (clock() - hfst::pmatch::timer) /
