@@ -11,12 +11,13 @@
 #include "xre_utils.h"
 #include "HfstTransducer.h"
 #include "HfstXeroxRules.h"
+#include "XreCompiler.h"
 
 using std::string;
 using std::map;
 using std::ostringstream;
 
-class yy_buffer_state;
+struct yy_buffer_state;
 typedef yy_buffer_state * YY_BUFFER_STATE;
 typedef void * yyscan_t;
 extern int xreparse(yyscan_t);
@@ -30,10 +31,21 @@ namespace hfst {
   namespace xre {
     extern unsigned int cr; // number of characters read, defined in XreCompiler.cc
     bool allow_extra_text_at_end = false;
-    extern std::string error_message;
+    extern std::ostream * error_;
+    extern bool verbose_;
   }
 }
 
+std::ostream * xreerrstr()
+{
+  return hfst::xre::XreCompiler::get_stream(hfst::xre::error_);
+}
+
+void xreflush(std::ostream * os)
+{
+  hfst::xre::XreCompiler::flush(os);
+} 
+
 int xreerror(yyscan_t scanner, const char* msg)
 { 
   char buffer [1024];
@@ -50,8 +62,13 @@ int xreerror(yyscan_t scanner, const char* msg)
                   hfst::xre::data, xreget_text(scanner));
     }
 
-  buffer[1023] == '\0';
-  hfst::xre::error_message = std::string(buffer);
+  buffer[1023] = '\0';
+  if (hfst::xre::verbose_)
+    {
+      std::ostream * err = xreerrstr();
+      *(err) << std::string(buffer);
+      xreflush(err);
+    }
   return 0;
 }
 
@@ -60,8 +77,10 @@ xreerror(const char *msg)
 {
   char buffer [1024];
   int n = sprintf(buffer, "*** xre parsing failed: %s\n", msg);
-  buffer[1023] == '\0';
-  hfst::xre::error_message = std::string(buffer);
+  buffer[1023] = '\0';
+  std::ostream * err = xreerrstr();
+  *err << std::string(buffer);
+  xreflush(err);
   return 0;
 }
 
@@ -84,8 +103,7 @@ size_t len;
   bool expand_definitions=false;
   bool harmonize_=true;
   bool harmonize_flags_=false;
-  bool verbose_=false;
-  FILE * warning_stream=NULL;
+  //bool verbose_=false;
 
   std::string substitution_function_symbol;
 
@@ -260,6 +278,8 @@ get_quoted(const char *s)
 char*
 parse_quoted(const char *s)
 {
+  std::ostream * err = xreerrstr();
+
     char* quoted = get_quoted(s);
     char* rv = static_cast<char*>(malloc(sizeof(char)*strlen(quoted) + 1)); // added + 1
     char* p = quoted;
@@ -284,8 +304,8 @@ parse_quoted(const char *s)
               case '5':
               case '6':
               case '7':
-                fprintf(stderr, "*** XRE unimplemented: "
-                        "parse octal escape in %s", p);
+                *err << "*** XRE unimplemented: parse octal escape in " << std::string(p);
+                xreflush(err);
                 *r = '\0';
                 p = p + 5;
                 break;
@@ -320,7 +340,8 @@ parse_quoted(const char *s)
                 p = p + 2;
                 break;
               case 'u':
-                fprintf(stderr, "Unimplemented: parse unicode escapes in %s", p);
+                *err << "Unimplemented: parse unicode escapes in " << std::string(p);
+                xreflush(err);
                 *r = '\0';
                 r++;
                 p = p + 6;
@@ -340,8 +361,10 @@ parse_quoted(const char *s)
                       }
                     else
                       {
-                        fprintf(stderr, "*** XRE unimplemented: "
-                                "parse \\x%d\n", i);
+                        *err << "*** XRE unimplemented: parse \\x" << i << std::endl;
+                        xreflush(err);
+                        //fprintf(stderr, "*** XRE unimplemented: "
+                        //        "parse \\x%d\n", i);
                         *r = '\0';
                       }
                     r++;
@@ -350,7 +373,9 @@ parse_quoted(const char *s)
                    break;
                 }
               case '\0':
-                fprintf(stderr, "End of line after \\ escape\n");
+                *err << "End of line after \\ escape" << std::endl;
+                xreflush(err);
+                //fprintf(stderr, "End of line after \\ escape\n");
                 *r = '\0';
                 r++;
                 p++;
@@ -543,7 +568,10 @@ bool is_valid_function_call
   if (name2xre == function_definitions.end() || 
       name2args == function_arguments.end())
     {
-      fprintf(stderr, "No such function defined: '%s'\n", name);
+      std::ostream * err = xreerrstr();
+      *err << "No such function defined: '" << name << "'" << std::endl;
+      xreflush(err);
+      //fprintf(stderr, "No such function defined: '%s'\n", name);
       return false;
     }
 
@@ -551,8 +579,12 @@ bool is_valid_function_call
 
   if ( number_of_args != args->size())
     {
-      fprintf(stderr, "Wrong number of arguments: function '%s' expects %i, %i given\n", 
-              name, (int)number_of_args, (int)args->size());
+      std::ostream * err = xreerrstr();
+      *err << "Wrong number of arguments: function '" << name << "' expects " 
+                           << (int)number_of_args << ", " << (int)args->size() << " given" << std::endl;
+      xreflush(err);
+        //fprintf(stderr, "Wrong number of arguments: function '%s' expects %i, %i given\n", 
+        //       name, (int)number_of_args, (int)args->size());
       return false;
     }
 
@@ -570,12 +602,11 @@ const char * get_function_xre(const char * name)
   return it->second.c_str();
 }
 
-void define_function_args(const char * name, const std::vector<HfstTransducer> * args)
+bool define_function_args(const char * name, const std::vector<HfstTransducer> * args)
 {
   if (! is_valid_function_call(name, args))
     {
-      fprintf(stderr, "Could not define function args\n");
-      exit(1);
+      return false;
     }
   unsigned int arg_number = 1;
   for (std::vector<HfstTransducer>::const_iterator it = args->begin();
@@ -589,6 +620,7 @@ void define_function_args(const char * name, const std::vector<HfstTransducer> *
       //std::cerr << *it << std::endl;
       arg_number++;
     }
+  return true;
 }
 
 void undefine_function_args(const char * name)
@@ -952,14 +984,16 @@ xfst_label_to_transducer(const char* input, const char* output)
     if (!verbose_)
       return;
     
-    fprintf(warning_stream, "%s", msg);
+    std::ostream * err = xreerrstr();
+    *err << msg;
+    xreflush(err);
   }
 
 void warn_about_xfst_special_symbol(const char * symbol)
 {
   if (strcmp("all", symbol) == 0) {
     if (verbose_) {
-      fprintf(warning_stream, "warning: symbol 'all' has no special meaning in hfst\n"); }
+      warn("warning: symbol 'all' has no special meaning in hfst\n"); }
     return;
   }
 
@@ -976,7 +1010,9 @@ void warn_about_xfst_special_symbol(const char * symbol)
     return;
   if (!verbose_)
     return;
-  fprintf(warning_stream, "warning: '%s' is an ordinary symbol in hfst\n", symbol);
+  std::ostream * err = xreerrstr();
+  *err << "warning: '" << symbol << " ' is an ordinary symbol in hfst" << std::endl;
+  xreflush(err);
 }
 
 void warn_about_hfst_special_symbol(const char * symbol)
@@ -997,7 +1033,9 @@ void warn_about_hfst_special_symbol(const char * symbol)
     return;
   if (!verbose_)
     return;
-  fprintf(warning_stream, "warning: '%s' is not an ordinary symbol in hfst\n", symbol);
+  std::ostream * err = xreerrstr();
+  *err << "warning: '" << symbol << "' is not an ordinary symbol in hfst" << std::endl;
+  xreflush(err);
 }
 
 void warn_about_special_symbols_in_replace(HfstTransducer * t)
@@ -1005,6 +1043,8 @@ void warn_about_special_symbols_in_replace(HfstTransducer * t)
   if (!verbose_)
     return;
 
+  std::ostream * err = xreerrstr();  
+
   StringSet alphabet = t->get_alphabet();
   for (StringSet::const_iterator it = alphabet.begin(); 
        it != alphabet.end(); it++)
@@ -1013,11 +1053,11 @@ void warn_about_special_symbols_in_replace(HfstTransducer * t)
           *it != hfst::internal_epsilon &&
           *it != hfst::internal_unknown &&
           *it != hfst::internal_identity)
-        {         
-          fprintf(warning_stream, "warning: using special symbol '%s' in replace rule, "
-                  "use substitute instead\n", it->c_str());
+        {  
+          *err << "warning: using special symbol '" << *it << "' in replace rule, use substitute instead" << std::endl;
         }
     }
+  xreflush(err);
 }
 
 bool has_non_identity_pairs(const HfstTransducer * t)
