@@ -224,6 +224,13 @@ int main (int argc, char * argv[])
           verbose_printf("Requiring foma support from library");
 #ifndef HAVE_FOMA
           error(EXIT_FAILURE, 0, "Required foma support not present");
+#endif
+        }
+      else if ((*f == "xfsm") || (*f == "XFSM") || (*f == "HAVE_XFSM"))
+        {
+          verbose_printf("Requiring xfsm support from library");
+#ifndef HAVE_XFSM
+          error(EXIT_FAILURE, 0, "Required xfsm support not present");
 #endif
         }
       else if ((*f == "openfst") || (*f == "OPENFST") || (*f == "HAVE_OPENFST"))
@@ -266,6 +273,9 @@ int main (int argc, char * argv[])
 #if HAVE_FOMA
   verbose_printf("foma supported\n");
 #endif
+#if HAVE_XFSM
+  verbose_printf("xfsm supported\n");
+#endif
 #if USE_GLIB_UNICODE
   verbose_printf("Unicode support: glib\n");
 #elif USE_ICU_UNICODE
